/*=========================================================================

Program:   Orbit Visualization Tool
Source:    $Source: /stor/devel/ovt2g/ovt/util/TimeFormat.java,v $
Date:      $Date: 2002/06/30 17:10:25 $
Version:   $Revision: 2.3 $


Copyright (c) 2000-2002 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev,
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 
yyyy ddd


hh.hhhh
hh:mm:ss
hh:mm

 *
 * Created on 9  2000 ., 18:56
 */

package ovt.util;

import ovt.datatype.*;
import ovt.beans.*;
import ovt.object.*;
import ovt.interfaces.*;

import java.beans.*;
import java.text.*;
import java.lang.*;
import java.util.*;

/**
 *
 * @author  ko
 * @version 
 */
public class TimeFormat extends OVTObject {
    /** date format : <CODE>yyyy-mm-dd</CODE> */
    public final static int YYYY_MM_DD = 1; 
    /** date format : <CODE>yyyy/mm/dd</CODE> */
    public final static int YYYY_MM_DD_SLASH = 2; 
    /** date format : <CODE>yyyy ddd</CODE> */
    public final static int YYYY_DDD = 3;
    // hours format
    /** hours format : <CODE>hh:mm:ss</CODE> */
    public final static int HH_MM_SS = 8;
    /** hours format : <CODE>hh:mm</CODE> */
    public final static int HH_MM = 9; 
    /** hours format : <CODE>hh.hhhh</CODE> */
    public final static int HH_HHHH = 10;
    /** hours format : <CODE>sssss.ss</CODE> */
    public final static int SSSSS_SS = 11;
    
    /** Holds value of dateFormat. By default {@link YYYY_MM_DD}. */
    private int dateFormat = YYYY_MM_DD;
    /** Holds value of property hoursFormat. */
    private int hoursFormat = HH_MM_SS;
    
    private DecimalFormat DDD_format = new DecimalFormat("000");
    private DecimalFormat HH_HHHH_format = new DecimalFormat("00.####");
    private DecimalFormat SSSSS_SS_format = new DecimalFormat("00000.##");
    /** Creates new TimeFormat */
    public TimeFormat() {
    }
    
    public String format(ovt.datatype.Time time) {
        String date = "", hours = "";
        switch (dateFormat) {
            case YYYY_MM_DD : date = time.getAsText(Time.YEAR)+"-"+
                                 time.getAsText(Time.MONTH)+"-"+
                                 time.getAsText(Time.DAY);
                                 break;
            case YYYY_MM_DD_SLASH : date = time.getAsText(Time.YEAR)+"/"+
                                 time.getAsText(Time.MONTH)+"/"+
                                 time.getAsText(Time.DAY);
                                 break;
            // this is not already implemented
            case YYYY_DDD : date = time.getAsText(Time.YEAR) + " " +
                                    DDD_format.format(time.getDayOfTheYear());
                                 break;
        }
        switch (hoursFormat) {
            case SSSSS_SS : double s = (time.getHour()*60 + time.getMinutes())*60 + time.getSeconds() + time.getMsec()/60.;
                            hours = SSSSS_SS_format.format(s);
                            break;
            case HH_HHHH : double h = time.getHour() + (time.getMinutes() + (time.getSeconds() + time.getMsec()/60.)/60.)/60.;
                            hours = HH_HHHH_format.format(h);
                            break;
            case HH_MM_SS: hours = time.getAsText(Time.HOUR)+":"+
                                   time.getAsText(Time.MINUTE)+":"+
                                   time.getAsText(Time.SECOND);
                                   break;
            case HH_MM : hours = time.getAsText(Time.HOUR)+":"+
                                   time.getAsText(Time.MINUTE);
                                    break;
        }
        return date + " " + hours;
    }
    
    public StringBuffer format(Object obj, StringBuffer buf, FieldPosition pos) {
        return new StringBuffer("a");
    }
    
    /** Getter for property dateFormat.
     * @return Value of property dateFormat.
 */
    public int getDateFormat() {
        return dateFormat;
    }
    
    public String getDateFormatString() {
        return getFormat(dateFormat);
    }
    /** Setter for property dateFormat.
     * @param dateFormat New value of property dateFormat.
 */
    public void setDateFormat(int dateFormat) throws IllegalArgumentException {
        if (dateFormat != YYYY_DDD  &&  dateFormat != YYYY_MM_DD  && dateFormat != YYYY_MM_DD_SLASH)
            throw new IllegalArgumentException("Invalid format type '"+ dateFormat +"'");
        int oldDateFormat = this.dateFormat;
        this.dateFormat = dateFormat;
        firePropertyChange ("dateFormat", new Integer (oldDateFormat), new Integer (dateFormat));
    }
    
    /** Getter for property hoursFormat.
     * @return Value of property hoursFormat.
 */
    public int getHoursFormat() {
        return hoursFormat;
    }
    
    public String getHoursFormatString() {
        return getFormat(hoursFormat);
    }
    
    /** Setter for property hoursFormat.
     * @param hoursFormat New value of property hoursFormat.
 */
    public void setHoursFormat(int hoursFormat) {
        if ( hoursFormat != HH_HHHH  &&  hoursFormat != HH_MM  
                && hoursFormat != HH_MM_SS && hoursFormat != SSSSS_SS )
            throw new IllegalArgumentException("Invalid format type '"+ hoursFormat +"'");
        int oldHoursFormat = this.hoursFormat;
        this.hoursFormat = hoursFormat;
        firePropertyChange ("hoursFormat", new Integer (oldHoursFormat), new Integer (hoursFormat));
    }
    
    
    public static String getFormat(int type) {
        switch (type) {
            case YYYY_MM_DD :  return "yyyy-mm-dd";
            case YYYY_MM_DD_SLASH : return "yyyy/mm/dd";
            case YYYY_DDD :  return "yyyy ddd";
            case HH_MM_SS :  return "hh:mm:ss";
            case HH_MM    :  return "hh:mm";
            case HH_HHHH  :  return "hh.hhhh";
            case SSSSS_SS :  return "sssss.ss";
        }
        throw new IllegalArgumentException("Invalid format type '"+ type +"'");
    }
    
    public static int[] getDateFormats() {
        return new int[]{ YYYY_MM_DD, YYYY_MM_DD_SLASH, YYYY_DDD };
    }
    
    public static String[] getDateFormatNames() {
        int[] formats = getDateFormats();
        String[] res = new String[formats.length];
        for (int i=0; i<formats.length; i++)
            res[i] = getFormat(formats[i]);
        return res;
    }
    
    public static int[] getHoursFormats() {
        return new int[]{ HH_MM, HH_MM_SS, HH_HHHH, SSSSS_SS };
    }
    
    public static String[] getHoursFormatNames() {
        int[] formats = getHoursFormats();
        String[] res = new String[formats.length];
        for (int i=0; i<formats.length; i++)
            res[i] = getFormat(formats[i]);
        return res;
    }
    
    public DoubleAndInteger parseMjd(String str) throws NumberFormatException {
        TimeAndStringLength a = parse(str);
        return new DoubleAndInteger(a.time.getMjd(), a.length);
    }
    
    public TimeAndStringLength parse(String str) throws NumberFormatException {
        int year=0, month=0, day=0, hour=0, mins=0, sec=0, msec=0;
        int offset = 11;
        int length = -1;
        try {
        switch (dateFormat) {
            case YYYY_MM_DD:
            case YYYY_MM_DD_SLASH:
                year = Integer.valueOf(str.substring(0,4)).intValue();
                month = Integer.valueOf(str.substring(5,7)).intValue();
                day = Integer.valueOf(str.substring(8,10)).intValue();
                break;
            case YYYY_DDD:
                year = Integer.valueOf(str.substring(0,4)).intValue();
                month = 0; // needs a check
                day  = Integer.valueOf(str.substring(5,8)).intValue();
                offset = 9;
                break;
        }
        length = offset;
        switch (hoursFormat) {
            case HH_MM_SS:
                StringTokenizer st = new StringTokenizer(str.substring(offset+6, str.length()));
                //int endIndex = StringUtils.doubleEndsAt(str, offset+6);
                String s = st.nextToken();
                //Log.log("secs='"+s+"'");
                double temp = new Double(s).doubleValue();
                sec = (int)temp;
                msec = (int)(temp - sec)*1000; // - not shure, may be wrong ;)?
                length += s.length();
            case HH_MM:
                hour = Integer.valueOf(str.substring(offset, offset+2)).intValue();
                mins = Integer.valueOf(str.substring(offset+3, offset+5)).intValue();
                if (length != -1) length += 5;
                break;
            case HH_HHHH:
                StringTokenizer st2 = new StringTokenizer(str.substring(offset, str.length()));
                String s2 = st2.nextToken();
                double temp2 = new Double(s2).doubleValue();
                hour = (int)temp2;
                temp = (temp2 - hour)*60;
                mins = (int)temp2;
                sec = (int)(temp2 - mins)*60;
                length += s2.length();
                break;
            case SSSSS_SS:
                StringTokenizer st3 = new StringTokenizer(str.substring(offset, str.length()));
                String s3 = st3.nextToken();
                double temp3 = new Double(s3).doubleValue();
                sec = (int)temp3;
                msec = (int)(temp3 - sec)*1000;
                length += s3.length();
                break;    
        }
        } catch ( NoSuchElementException e2) {
            throw new NumberFormatException(e2.getMessage());
        }
        try {
            return new TimeAndStringLength(new Time(year, month, day, hour, mins, sec, msec), length);
        } catch (IllegalArgumentException ill_a_e) {
            throw new NumberFormatException(ill_a_e.getMessage());
        }
    }
    
    
    /*public Time parse(StringBuffer str) throws NumberFormatException {
        int year=0, month=0, day=0, hour=0, mins=0, sec=0;
        try {
        switch (dateFormat) {
            case YYYY_MM_DD:
            case YYYY_MM_DD_SLASH:
                year = Integer.valueOf(str.substring(0,4)).intValue();
                month = Integer.valueOf(str.substring(5,7)).intValue();
                day = Integer.valueOf(str.substring(8,10)).intValue();
                str.delete(0, 11);
                break;
            case YYYY_DDD:
                year = Integer.valueOf(str.substring(0,4)).intValue();
                month = 0; // needs a check
                day  = Integer.valueOf(str.substring(5,8)).intValue();
                str.delete(0, 9);
                break;
        }
        switch (hoursFormat) {
            case HH_MM_SS:
                StringTokenizer st = new StringTokenizer(str.substring(6, str.length()));
                //int endIndex = StringUtils.doubleEndsAt(str, offset+6);
                String s = st.nextToken();
                sec = Integer.valueOf(s).intValue();
                str.delete(5, s.length());
            case HH_MM:
                hour = Integer.valueOf(str.substring(0, 2)).intValue();
                mins = Integer.valueOf(str.substring(3, 5)).intValue();
                str.delete(
                break;
            case HH_HHHH:
                StringTokenizer st2 = new StringTokenizer(str.substring(offset, str.length()));
                hour = Integer.valueOf(st2.nextToken()).intValue();
                break;
            case SSSSS_SS:
                StringTokenizer st3 = new StringTokenizer(str.substring(offset, str.length()));
                sec = Integer.valueOf(st3.nextToken()).intValue();
                break;    
        }
        } catch ( NoSuchElementException e2) {
            throw new NumberFormatException(e2.getMessage());
        }
        return new Time(year, month, day, hour, mins, sec, 0);
    }*/
    
    public static int guessDateFormat(String str) {
        
        if (str.indexOf('-') == 4 && str.indexOf('-', str.indexOf('-')+1) == 7) {
            if (isInteger(str, 0, 4) && isInteger(str, 5, 7) && isInteger(str, 8, 10))
                return YYYY_MM_DD;
            else 
                throw new NumberFormatException("Wrong yyyy-mm-dd format ("+str+")");
        }
        if (str.indexOf('/') == 4 && str.indexOf('/', str.indexOf('/')+1) == 7) {
            if (isInteger(str, 0, 4) && isInteger(str, 5, 7) && isInteger(str, 8, 10))
                return YYYY_MM_DD_SLASH;
            else 
                throw new NumberFormatException("Wrong yyyy/mm/dd format ("+str+")");
        }
        if (str.indexOf(' ') == 4) {
            if (isInteger(str, 0, 4) && isInteger(str, 5, 8))
                return YYYY_DDD;
            else 
                throw new NumberFormatException("Wrong yyyy ddd format ("+str+")");
            
        }
        throw new NumberFormatException("Wrong date format ("+str+")");
    }
    
    /** Can distinguish hh:mm and hh:mm:ss formats*/
    public static int guessHoursFormat(String str) throws NumberFormatException {
        int semicIndex1 = str.indexOf(':');
        int semicIndex2 = str.indexOf(':', semicIndex1+1);
        
        if (semicIndex1 == 2) { // could be hh:mm or hh:mm:ss
            if (!isInteger(str, 0, 2)) 
                throw new NumberFormatException("Wrong hh format ("+str+")");
            if (!isInteger(str, 3, 5))
                throw new NumberFormatException("Wrong mm format ("+str+")");
            if (semicIndex2 == 5) {
                if (isInteger(str, 6, 8))
                    return HH_MM_SS;
                else 
                    throw new NumberFormatException("Wrong ss format ("+str+")");
            } else return HH_MM;
            
        }
        // left patterns: hh.hhhh and ssssss.ss
        // I don't know how to distinguish them..
        throw new NumberFormatException("Wrong time format ("+str+")");
        
    }
    
    /** Returns int[] = { dateFormat, hoursFormat } */
    public static int[] guessDateAndHoursFormat(String str) throws NumberFormatException {
        int dateFormat = guessDateFormat(str);
        int beginIndex = getFormat(dateFormat).length() +1;
        int hoursFormat = guessHoursFormat(str.substring(beginIndex, str.length()));
        return new int[]{ dateFormat, hoursFormat };
    }
    
    
    private static boolean isInteger(String str, int beginIndex, int endIndex) {
        try {
            new Integer(str.substring(beginIndex, endIndex));
        } catch (NumberFormatException e2) {
            return false;
        } catch (IndexOutOfBoundsException e3) {
            return false;
        }
        
        return true;
    }
    
    private static boolean isDouble(String str, int beginIndex, int endIndex) {
        try {
            new Double(str.substring(beginIndex, endIndex));
        } catch (NumberFormatException e2) {
            return false;
        } catch (IndexOutOfBoundsException e3) {
            return false;
        }
        return true;
    }
    
    public Descriptors getDescriptors() {
        if (descriptors == null) {
            try {
                descriptors = new Descriptors();
                
                // date format
                BasicPropertyDescriptor pd = new BasicPropertyDescriptor("dateFormat", this);
                pd.setDisplayName("Date format");
                pd.setMenuAccessible(false);
                GUIPropertyEditor editor = new ComboBoxPropertyEditor(pd, TimeFormat.getDateFormats(), TimeFormat.getDateFormatNames());
                pd.setPropertyEditor(editor);
                addPropertyChangeListener("dateFormat", editor);
                descriptors.put(pd);
                
                // hours format
                pd = new BasicPropertyDescriptor("hoursFormat", this);
                pd.setDisplayName("Hours format");
                pd.setMenuAccessible(false);
                editor = new ComboBoxPropertyEditor(pd, TimeFormat.getHoursFormats(), TimeFormat.getHoursFormatNames());
                pd.setPropertyEditor(editor);
                addPropertyChangeListener("hoursFormat", editor);
                descriptors.put(pd);
                
            } catch (IntrospectionException e2) {
                System.out.println(getClass().getName() + " -> " + e2.toString());
                System.exit(0);
            }
        }
        return descriptors;
    }
    
    public static void main(String[] args) {
        Time time = new Time("2001-02-03 10:00:36");
        TimeFormat tf = new TimeFormat();
        tf.setHoursFormat(tf.HH_HHHH);
        
        int[] f = TimeFormat.guessDateAndHoursFormat(args[0]);
        System.out.println(TimeFormat.getFormat(f[0])+" "+TimeFormat.getFormat(f[1]));
        String str = "123456";
        System.out.println(str.substring(0, 2));
    }
}

class TimeAndStringLength {

    public Time time = null;
    public int length = 0;
    public TimeAndStringLength(Time time, int length) {
        this.time = time;
        this.length = length;
    }
}
